@@ -47,6 +47,7 @@ NULL
 #' @rdname operator.operations
 #' @export
 Pr.mult <- function(obj, v, transpose = FALSE) {
+  orig_v <- v
   if (!inherits(obj, "rSPDEobj")) {
     stop("obj is not of class rSPDE.obj")
   }
@@ -65,13 +66,14 @@ Pr.mult <- function(obj, v, transpose = FALSE) {
   if (!transpose) {
     v <- obj$Pr.factors$Phi %*% v
   }
-  return(v)
+  return(return_same_input_type_matrix_vector(v,orig_v))
 }
 
 
 #' @rdname operator.operations
 #' @export
 Pr.solve <- function(obj, v, transpose = FALSE) {
+  orig_v <- v  
   if (!inherits(obj, "rSPDEobj")) {
     stop("obj is not of class rSPDE.obj")
   }
@@ -90,12 +92,13 @@ Pr.solve <- function(obj, v, transpose = FALSE) {
   if (transpose) {
     v <- solve(obj$Pr.factors$Phi, v)
   }
-  return(v)
+  return(return_same_input_type_matrix_vector(v,orig_v))
 }
 
 #' @rdname operator.operations
 #' @export
 Pl.mult <- function(obj, v, transpose = FALSE) {
+  orig_v <- v  
   if (!inherits(obj, "rSPDEobj")) {
     stop("obj is not of class rSPDE.obj")
   }
@@ -125,12 +128,13 @@ Pl.mult <- function(obj, v, transpose = FALSE) {
     v <- obj$C %*% v
   }
   v <- obj$Pl.factors$scaling * v
-  return(v)
+  return(return_same_input_type_matrix_vector(v,orig_v))
 }
 
 #' @rdname operator.operations
 #' @export
 Pl.solve <- function(obj, v, transpose = FALSE) {
+  orig_v <- v  
   if (!inherits(obj, "rSPDEobj")) {
     stop("obj is not of class rSPDE.obj")
   }
@@ -160,17 +164,18 @@ Pl.solve <- function(obj, v, transpose = FALSE) {
     v <- obj$Ci %*% v
   }
   v <- v / obj$Pl.factors$scaling
-  return(v)
+  return(return_same_input_type_matrix_vector(v,orig_v))
 }
 
 #' @rdname operator.operations
 #' @export
 Q.mult <- function(obj, v) {
+  orig_v <- v  
   if (inherits(obj, "rSPDEobj")) {
     v <- Pl.mult(obj, v)
     v <- obj$Ci %*% v
     v <- Pl.mult(obj, v, transpose = TRUE)
-    return(v)
+  return(return_same_input_type_matrix_vector(v,orig_v))
   } else if (inherits(obj, "CBrSPDEobj")) {
     Q.int <- obj$Q.int
     order_Q_int <- Q.int$order
@@ -182,7 +187,7 @@ Q.mult <- function(obj, v) {
         v <- Q.int %*% v
       }
     }
-    return(v)
+  return(return_same_input_type_matrix_vector(v,orig_v))
   } else {
     stop("obj is not of class rSPDEobj")
   }
@@ -191,11 +196,12 @@ Q.mult <- function(obj, v) {
 #' @rdname operator.operations
 #' @export
 Q.solve <- function(obj, v) {
+  orig_v <- v  
   if (inherits(obj, "rSPDEobj")) {
     v <- Pl.solve(obj, v, transpose = TRUE)
     v <- obj$C %*% v
     v <- Pl.solve(obj, v)
-    return(v)
+    return(return_same_input_type_matrix_vector(v,orig_v))
   } else if (inherits(obj, "CBrSPDEobj")) {
     Q.int <- obj$Q.int
     Q.frac <- obj$Q.frac
@@ -208,7 +214,7 @@ Q.solve <- function(obj, v) {
       }
     }
     v <- solve(Q.frac, prod_tmp)
-    return(v)
+    return(return_same_input_type_matrix_vector(v,orig_v))
   } else {
     stop("obj is not of class rSPDEobj nor CBrSPDEobj")
   }
@@ -217,6 +223,7 @@ Q.solve <- function(obj, v) {
 #' @rdname operator.operations
 #' @export
 Qsqrt.mult <- function(obj, v, transpose = FALSE) {
+  orig_v <- v
   if (!inherits(obj, "rSPDEobj")) {
     stop("obj is not of class rSPDE.obj")
   }
@@ -227,12 +234,13 @@ Qsqrt.mult <- function(obj, v, transpose = FALSE) {
     v <- Pl.mult(obj, v)
     v <- sqrt(obj$Ci) %*% v
   }
-  return(v)
+  return(return_same_input_type_matrix_vector(v,orig_v))
 }
 
 #' @rdname operator.operations
 #' @export
 Qsqrt.solve <- function(obj, v, transpose = FALSE) {
+  orig_v <- v
   if (!inherits(obj, "rSPDEobj")) {
     stop("obj is not of class rSPDE.obj")
   }
@@ -243,33 +251,31 @@ Qsqrt.solve <- function(obj, v, transpose = FALSE) {
     v <- sqrt(obj$C) %*% v
     v <- Pl.solve(obj, v)
   }
-  return(v)
+  return(return_same_input_type_matrix_vector(v,orig_v))
 }
 
 #' @rdname operator.operations
 #' @export
 Sigma.mult <- function(obj, v) {
+  orig_v <- v
   if (!inherits(obj, "rSPDEobj")) {
     stop("obj is not of class rSPDE.obj")
   }
   v <- Pr.mult(obj, v, transpose = TRUE)
   v <- Q.solve(obj, v)
   v <- Pr.mult(obj, v)
-  return(v)
+  return(return_same_input_type_matrix_vector(v,orig_v))
 }
 
 #' @rdname operator.operations
 #' @export
 Sigma.solve <- function(obj, v) {
+  orig_v <- v
   if (!inherits(obj, "rSPDEobj")) {
     stop("obj is not of class rSPDE.obj")
   }
   v <- Pr.solve(obj, v)
   v <- Q.mult(obj, v)
   v <- Pr.solve(obj, v, transpose = TRUE)
-  return(v)
-}
-
-#' @rdname operator.operations
-#' @export
-#'
+  return(return_same_input_type_matrix_vector(v,orig_v))
+}
\ No newline at end of file
